@@ -6,43 +6,62 @@
  */
 
 #include "FlatpakNotifier.h"
+#include "libdiscover_backend_flatpak_debug.h"
 
 #include <glib.h>
 
-#include <QDebug>
 #include <QFutureWatcher>
 #include <QTimer>
 #include <QtConcurrentRun>
 
-static void installationChanged(GFileMonitor *monitor, GFile *child, GFile *other_file, GFileMonitorEvent event_type, gpointer self)
+using namespace std::chrono_literals;
+
+static void installationChanged(GFileMonitor *monitor, GFile *child, GFile *other_file, GFileMonitorEvent event_type, gpointer data)
 {
     Q_UNUSED(monitor);
     Q_UNUSED(child);
     Q_UNUSED(other_file);
     Q_UNUSED(event_type);
 
-    FlatpakNotifier::Installation *installation = (FlatpakNotifier::Installation *)self;
-    if (!installation)
+    FlatpakNotifier *notifier = (FlatpakNotifier *)data;
+    if (!notifier)
         return;
 
-    FlatpakNotifier *notifier = installation->m_notifier;
-    notifier->loadRemoteUpdates(installation);
+    for (const auto &installation : notifier->m_installations) {
+        if (installation->m_monitor == monitor) {
+            notifier->loadRemoteUpdates(installation);
+        }
+    }
 }
 
 FlatpakNotifier::FlatpakNotifier(QObject *parent)
     : BackendNotifierModule(parent)
-    , m_user(this)
-    , m_system(this)
     , m_cancellable(g_cancellable_new())
 {
     QTimer *dailyCheck = new QTimer(this);
-    dailyCheck->setInterval(24 * 60 * 60 * 1000); // refresh at least once every day
+    dailyCheck->setInterval(24h); // refresh at least once every day
     connect(dailyCheck, &QTimer::timeout, this, &FlatpakNotifier::recheckSystemUpdateNeeded);
+
+    g_autoptr(GError) error = nullptr;
+    g_autoptr(GPtrArray) installations = flatpak_get_system_installations(m_cancellable, &error);
+    if (error) {
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to call flatpak_get_system_installations:" << error->message;
+    }
+    for (uint i = 0; installations && i < installations->len; i++) {
+        auto installation = FLATPAK_INSTALLATION(g_ptr_array_index(installations, i));
+        m_installations << std::make_shared<Installation>(this, installation);
+    }
+
+    if (auto user = flatpak_installation_new_user(m_cancellable, &error)) {
+        m_installations << std::make_shared<Installation>(this, user);
+    }
 }
 
-FlatpakNotifier::Installation::Installation(FlatpakNotifier *notifier)
+FlatpakNotifier::Installation::Installation(FlatpakNotifier *notifier, FlatpakInstallation *installation)
     : m_notifier(notifier)
+    , m_installation(installation)
 {
+    g_object_ref(installation);
 }
 
 FlatpakNotifier::Installation::~Installation()
@@ -60,19 +79,15 @@ FlatpakNotifier::~FlatpakNotifier()
 
 void FlatpakNotifier::recheckSystemUpdateNeeded()
 {
-    g_autoptr(GError) error = nullptr;
+    setupFlatpakInstallations();
 
-    // Load flatpak installation
-    if (!setupFlatpakInstallations(&error)) {
-        qWarning() << "Failed to setup flatpak installations: " << error->message;
-    } else {
-        // Load updates from remote repositories
-        loadRemoteUpdates(&m_system);
-        loadRemoteUpdates(&m_user);
+    // Load updates from remote repositories
+    for (auto &installation : std::as_const(m_installations)) {
+        loadRemoteUpdates(installation);
     }
 }
 
-void FlatpakNotifier::onFetchUpdatesFinished(Installation *installation, bool hasUpdates)
+void FlatpakNotifier::onFetchUpdatesFinished(const std::shared_ptr<Installation> &installation, bool hasUpdates)
 {
     if (installation->m_hasUpdates == hasUpdates) {
         return;
@@ -85,8 +100,9 @@ void FlatpakNotifier::onFetchUpdatesFinished(Installation *installation, bool ha
     }
 }
 
-void FlatpakNotifier::loadRemoteUpdates(Installation *installation)
+void FlatpakNotifier::loadRemoteUpdates(const std::shared_ptr<Installation> &installation)
 {
+    Q_ASSERT(installation->m_installation);
     auto fw = new QFutureWatcher<bool>(this);
     connect(fw, &QFutureWatcher<bool>::finished, this, [this, installation, fw]() {
         onFetchUpdatesFinished(installation, fw->result());
@@ -99,7 +115,7 @@ void FlatpakNotifier::loadRemoteUpdates(Installation *installation)
         bool hasUpdates = false;
 
         if (!fetchedUpdates) {
-            qWarning() << "Failed to get list of installed refs for listing updates: " << localError->message;
+            qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to get list of installed refs for listing updates: " << localError->message;
             return false;
         }
         for (uint i = 0; !hasUpdates && i < fetchedUpdates->len; i++) {
@@ -119,35 +135,30 @@ void FlatpakNotifier::loadRemoteUpdates(Installation *installation)
 
 bool FlatpakNotifier::hasUpdates()
 {
-    return m_system.m_hasUpdates || m_user.m_hasUpdates;
+    return std::ranges::any_of(m_installations, [](const auto &installation) {
+        return installation->m_hasUpdates;
+    });
 }
 
-bool FlatpakNotifier::Installation::ensureInitialized(std::function<FlatpakInstallation *()> func, GCancellable *cancellable, GError **error)
+bool FlatpakNotifier::Installation::ensureInitialized(GCancellable *cancellable)
 {
-    if (!m_installation) {
-        m_installation = func();
-        m_monitor = flatpak_installation_create_monitor(m_installation, cancellable, error);
-        g_signal_connect(m_monitor, "changed", G_CALLBACK(installationChanged), this);
+    if (!m_monitor) {
+        g_autoptr(GError) error = nullptr;
+        m_monitor = flatpak_installation_create_monitor(m_installation, cancellable, &error);
+        if (m_monitor) {
+            g_signal_connect(m_monitor, "changed", G_CALLBACK(installationChanged), m_notifier);
+        } else {
+            qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Failed to setup flatpak installation: " << error->message;
+        }
     }
     return m_installation && m_monitor;
 }
 
-bool FlatpakNotifier::setupFlatpakInstallations(GError **error)
+void FlatpakNotifier::setupFlatpakInstallations()
 {
-    if (!m_system.ensureInitialized(
-            [this, error] {
-                return flatpak_installation_new_system(m_cancellable, error);
-            },
-            m_cancellable,
-            error))
-        return false;
-    if (!m_user.ensureInitialized(
-            [this, error] {
-                return flatpak_installation_new_user(m_cancellable, error);
-            },
-            m_cancellable,
-            error))
-        return false;
-
-    return true;
+    for (auto &installation : m_installations) {
+        installation->ensureInitialized(m_cancellable);
+    }
 }
+
+#include "moc_FlatpakNotifier.cpp"
